// BoardSolitaire2.java

import ch.aplu.jgamegrid.*;
import java.util.*;

public class BoardSolitaire2 extends GameGrid implements GGMouseListener
{

  final int DOWN = 0;
  final int UP = 1;
  final int restart = 5000; //restart game after ... milliseconds
  private ArrayList<Location> locations = new ArrayList<Location>();

  public BoardSolitaire2()
  {
    super(7, 7, 70, null, "sprites/boardSolitaire.png", false);

    loadMarbleLocations();
    loadMarbles();

    addMouseListener(this, GGMouse.lPress);
    show();
  }

  //set locations where a marble is set at the start
  private void loadMarbleLocations()
  {
    for (int y = 0; y < 7; y++)
    {
      if (y < 2 || y > 4)
      {
        for (int x = 2; x < 5; x++)
          locations.add(new Location(x, y));
      }
      else
      {
        for (int x = 0; x < 7; x++)
          locations.add(new Location(x, y));
      }
    }
  }

  //set marbles on the board
  private void loadMarbles()
  {
    for (int l = 0; l < locations.size(); l++)
    {
      if (!locations.get(l).equals(new Location(3, 3)))
        addActor(new Marble(DOWN), locations.get(l));
    }
  }
  //if mouse is pressed

  public boolean mouseEvent(GGMouse mouse)
  {
    Location location = toLocationInGrid(mouse.getX(), mouse.getY());
    Actor marble = getOneActorAt(location, Marble.class);
    Actor oneUp = getOneActorUp();

    //if there is a marble
    if (marble != null && isPossibleLocation(location))
    {
      //if no marble has been selected before, lift it up
      if (marble.getIdVisible() == DOWN && oneUp == null)
        marble.show(UP);
      //if one marble has been selected before and an other one is selected, set
      //the old one down and lift the new one up
      else if (oneUp != null)
      {
        marble.show(UP);
        oneUp.show(DOWN);
      }
      //if the selected marble has already been lifted up and selected again,
      //set it back down
      else
        marble.show(DOWN);
    }
    //if selected location is empty and a marble has been selected before check
    //if the marble between can be jumped and removed
    else if (oneUp != null && marble == null && isPossibleLocation(location))
    {
      Location upLoc = oneUp.getLocation();
      Double dir = upLoc.getDirectionTo(location);
      int distance = upLoc.getDistanceTo(location);

      if (distance == 2)
      {
        Actor between = getOneActorAt(upLoc.getNeighbourLocation(dir));

        if (between != null)
        {
          between.removeSelf();
          oneUp.setLocation(location);
          oneUp.show(DOWN);

          isGameOver();
        }
      }
    }
    refresh();
    return true;
  }

  //check if location is inside board
  private boolean isPossibleLocation(Location loc)
  {
    boolean isPossible = false;

    for (Location l : locations)
    {
      if (l.equals(loc))
        isPossible = true;
    }
    return isPossible;
  }

  //get marble which has been selected to be played
  private Actor getOneActorUp()
  {
    Actor oneUp = null;

    ArrayList<Actor> marbles = getActors(Marble.class);
    for (Actor m : marbles)
    {
      if (m.getIdVisible() == UP)
        oneUp = m;
    }

    return oneUp;
  }

  //check if game is won or lost
  private void isGameOver()
  {
    ArrayList<Location> leftOvers = getOccupiedLocations();
    //One left => you win
    if (leftOvers.size() == 1)
    {
      addActor(new Actor("sprites/you_win.gif"), new Location(3, 3));
      restart();
    }
    else
    {
      boolean gameover = true;

      //more than one left, but without neighbours => you loose
      for (Location lO : leftOvers)
      {
        ArrayList<Location> neighbours = lO.getNeighbourLocations(1);
        for (Location n : neighbours)
        {
          if (getOneActorAt(n) != null)
            gameover = false;
        }
      }

      if (gameover)
      {
        addActor(new Actor("sprites/gameover.gif"), new Location(3, 3));
        restart();
      }
    }
  }

  //restart game
  private void restart()
  {
    delay(restart);
    removeAllActors();
    loadMarbles();
  }

  public static void main(String[] args)
  {
    new BoardSolitaire2();
  }
}

//------------------------------------------------------------------------------
class Marble extends Actor
{

  public Marble(int imgID)
  {
    super("sprites/marble.png", 2);
    show(imgID);
  } 
}
